import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import { endsWith as _endsWith, includes as _includes, isChineseIDCardNumber, isCuid, isCuid2, isEmail, isPossibleChineseMobilePhoneNumber, isString, isUrl, startsWith as _startsWith } from "../utils/index.js";
import { VaeLocale } from "./VaeLocale.js";
import { VaeSchema } from "./VaeSchema.js";
export var VaeStringSchema = /*#__PURE__*/function (_VaeSchema) {
  _inheritsLoose(VaeStringSchema, _VaeSchema);
  function VaeStringSchema(message) {
    var _this;
    if (message === void 0) {
      message = VaeLocale.string.type;
    }
    _this = _VaeSchema.call(this, {
      type: 'string'
    }) || this;
    _this.transform(function (v) {
      return typeof v === 'number' ? String(v) : v;
    }).check({
      fn: isString,
      message: message
    });
    return _this;
  }
  var _proto = VaeStringSchema.prototype;
  _proto.min = function min(value, message) {
    if (message === void 0) {
      message = VaeLocale.string.min;
    }
    return this.check({
      fn: function fn(v) {
        return v.length >= value;
      },
      message: message,
      messageParams: {
        min: value
      },
      tag: 'min'
    });
  };
  _proto.max = function max(value, message) {
    if (message === void 0) {
      message = VaeLocale.string.max;
    }
    return this.check({
      fn: function fn(v) {
        return v.length <= value;
      },
      message: message,
      messageParams: {
        max: value
      },
      tag: 'max'
    });
  };
  _proto.length = function length(value, message) {
    if (message === void 0) {
      message = VaeLocale.string.length;
    }
    return this.check({
      fn: function fn(v) {
        return v.length === value;
      },
      message: message,
      messageParams: {
        length: value
      },
      tag: 'length'
    });
  };
  _proto.email = function email(message) {
    if (message === void 0) {
      message = VaeLocale.string.email;
    }
    return this.check({
      fn: isEmail,
      message: message,
      tag: 'email'
    });
  };
  _proto.url = function url(message) {
    if (message === void 0) {
      message = VaeLocale.string.url;
    }
    return this.check({
      fn: isUrl,
      message: message,
      tag: 'url'
    });
  };
  _proto.regex = function regex(value, message) {
    if (message === void 0) {
      message = VaeLocale.string.regex;
    }
    return this.check({
      fn: function fn(v) {
        value.lastIndex = 0;
        return value.test(v);
      },
      message: message,
      messageParams: {
        regex: value
      },
      tag: 'regex'
    });
  };
  _proto.includes = function includes(value, message) {
    if (message === void 0) {
      message = VaeLocale.string.includes;
    }
    return this.check({
      fn: function fn(v) {
        return _includes(v, value);
      },
      message: message,
      messageParams: {
        includes: value
      },
      tag: 'includes'
    });
  };
  _proto.startsWith = function startsWith(value, message) {
    if (message === void 0) {
      message = VaeLocale.string.startsWith;
    }
    return this.check({
      fn: function fn(v) {
        return _startsWith(v, value);
      },
      message: message,
      messageParams: {
        startsWith: value
      },
      tag: 'startsWith'
    });
  };
  _proto.endsWith = function endsWith(value, message) {
    if (message === void 0) {
      message = VaeLocale.string.endsWith;
    }
    return this.check({
      fn: function fn(v) {
        return _endsWith(v, value);
      },
      message: message,
      messageParams: {
        endsWith: value
      },
      tag: 'endsWith'
    });
  };
  _proto.phoneNumber = function phoneNumber(message) {
    if (message === void 0) {
      message = VaeLocale.string.phoneNumber;
    }
    return this.check({
      fn: isPossibleChineseMobilePhoneNumber,
      message: message,
      tag: 'phoneNumber'
    });
  };
  _proto.idCardNumber = function idCardNumber(message) {
    if (message === void 0) {
      message = VaeLocale.string.idCardNumber;
    }
    return this.check({
      fn: isChineseIDCardNumber,
      message: message,
      tag: 'idCardNumber'
    });
  };
  _proto.cuid = function cuid(message) {
    if (message === void 0) {
      message = VaeLocale.string.cuid;
    }
    return this.check({
      fn: isCuid,
      message: message,
      tag: 'cuid'
    });
  };
  _proto.cuid2 = function cuid2(message) {
    if (message === void 0) {
      message = VaeLocale.string.cuid2;
    }
    return this.check({
      fn: isCuid2,
      message: message,
      tag: 'cuid2'
    });
  };
  _proto.trim = function trim() {
    this._options.stringTrim = true;
    return this;
  };
  _proto.emptyable = function emptyable() {
    this._options.stringEmptyable = true;
    return this;
  };
  return VaeStringSchema;
}(VaeSchema);